@@ -2,11 +2,16 @@
 
 # crosser.sh: Generic toolchain builder.
 #
-# (c) 2008-2009 Marko Lindqvist
+# (c) 2008-2010 Marko Lindqvist
 #
 # This program is licensed under Gnu General Public License version 2.
 
-MAINDIR=$(cd $(dirname $0) ; pwd)
+CROSSER_MAINDIR=$(cd $(dirname $0) ; pwd)
+
+if ! test -e "$CROSSER_MAINDIR/CrosserVersion"
+then
+  CROSSER_MAINDIR="/usr/share/crosser"
+fi
 
 STEP="setup"
 STEPADD="   "
@@ -24,9 +29,9 @@ if test "x$3" != "x" ; then
 else
   VERSIONSET="current"
 fi
-if test -e $MAINDIR/setups/$VERSIONSET.versions
+if test -e $CROSSER_MAINDIR/setups/$VERSIONSET.versions
 then
-  . $MAINDIR/setups/$VERSIONSET.versions
+  . $CROSSER_MAINDIR/setups/$VERSIONSET.versions
 else
   # Versions being unset do not prevent loading of build_setup.conf and helper.sh,
   # resulting environment would just be unusable for building.
@@ -35,10 +40,17 @@ else
   ERR_MSG="Cannot find versionset \"$VERSIONSET.versions\""
 fi
 
-. $MAINDIR/build_setup.conf
-. $MAINDIR/scripts/helpers.sh
-. $MAINDIR/scripts/packethandlers.sh
-. $MAINDIR/scripts/buildfuncs.sh
+. $CROSSER_MAINDIR/build_setup.conf
+. $CROSSER_MAINDIR/scripts/helpers.sh
+. $CROSSER_MAINDIR/scripts/packethandlers.sh
+. $CROSSER_MAINDIR/scripts/buildfuncs.sh
+
+# This must be after reading helpers.sh so that $CROSSER_VERSION is set
+if test "x$1" = "x-v" || test "x$1" = "x--version"
+then
+  echo "Crosser $CROSSER_VERSION"
+  exit 0
+fi
 
 if ! log_init
 then
@@ -67,7 +79,7 @@ fi
 if test "x$2" != "x" ; then
   STEPPARAM="$2"
 else
-  if test "x$FORCERM" != "xyes"
+  if test "x$CROSSER_FORCE" != "xyes"
   then
     ANSWER="unknown"
 
@@ -94,7 +106,7 @@ else
   STEPPARAM="native:sdl"
 fi
 
-. $MAINDIR/steps/stepfuncs.sh
+. $CROSSER_MAINDIR/steps/stepfuncs.sh
 
 STEPLIST="$(parse_steplist $STEPPARAM)"
 
@@ -105,13 +117,13 @@ then
 fi
 
 # Setup step variables
-. $MAINDIR/steps/stepset.sh
+. $CROSSER_MAINDIR/steps/stepset.sh
 
-if ! test -e "$MAINDIR/setups/$SETUP.conf" ; then
+if ! test -e "$CROSSER_MAINDIR/setups/$SETUP.conf" ; then
   log_error "Can't find setup \"$SETUP.conf\""
   exit 1
 fi
-source "$MAINDIR/setups/$SETUP.conf"
+source "$CROSSER_MAINDIR/setups/$SETUP.conf"
 
 if test "x$LIBC_MODE" = "xnone"
 then
@@ -126,8 +138,8 @@ else
   TARGET="$TARGET_ARCH-$TARGET_VENDOR-$TARGET_OS"
 fi
 
-if ! source $MAINDIR/setups/native.sh ; then
-  log_error "Failed to read $MAINDIR/setups/native.sh"
+if ! source $CROSSER_MAINDIR/setups/native.sh ; then
+  log_error "Failed to read $CROSSER_MAINDIR/setups/native.sh"
   exit 1
 fi
 NATIVE_ARCH="$TMP_ARCH"
@@ -399,7 +411,7 @@ setup_host_commands() {
   # Absolutely required commands
   HOST_COMMANDS_REQ="mkdir touch true false chmod rm which sed grep expr cat echo sort mv cp ln cmp test comm ls rmdir tr date uniq sleep diff basename dirname tail head env uname cut readlink od egrep fgrep wc make find pwd tar m4 awk getconf expand perl bison bzip2 flex makeinfo install whoami depmod wget pod2man msgfmt pkg-config sh glib-genmarshal hostname dnsdomainname mktemp"
   # Usefull commands
-  HOST_COMMANDS_TRY="dpkg-source md5sum gpg sha1sum sha256sum gzip gunzip patch"
+  HOST_COMMANDS_TRY="dpkg-source md5sum gpg gpgv sha1sum sha256sum gzip gunzip patch quilt bash getopt"
 
   log_write 1 "Setting up hostbin"
 
@@ -443,7 +455,7 @@ dummy_glibc_objects() {
        log_error "crt.o build failed"
        return 1
     fi
-    if ! $TARGET-gcc -c -shared -fPIC $MAINDIR/scripts/dummyclib.c -o libc.so \
+    if ! $TARGET-gcc -c -shared -fPIC $CROSSER_MAINDIR/scripts/dummyclib.c -o libc.so \
          2>>$LOGDIR/stderr.log >>$LOGDIR/stdout.log
     then
         log_error "Failed to build dummy libc.so"
@@ -499,7 +511,9 @@ prepare_binutils_src() {
 prepare_gcc_src() {
   if ! unpack_component gcc          $VERSION_GCC      ||
      ! unpack_component gmp          $VERSION_GMP      ||
-     ! unpack_component mpfr         $VERSION_MPFR
+     ! unpack_component mpfr         $VERSION_MPFR     ||
+     ! (is_smaller_version $VERSION_GCC 4.5.0 ||
+        unpack_component mpc          $VERSION_MPC )
   then
     log_error "Unpacking failed"
     exit 1
@@ -513,7 +527,9 @@ prepare_gcc_src() {
   fi
 
   if ! ln -s ../mpfr-$VERSION_MPFR $MAINSRCDIR/gcc-$VERSION_GCC/mpfr ||
-     ! ln -s ../gmp-$VERSION_GMP $MAINSRCDIR/gcc-$VERSION_GCC/gmp
+     ! ln -s ../gmp-$VERSION_GMP $MAINSRCDIR/gcc-$VERSION_GCC/gmp ||
+     ! (is_smaller_version $VERSION_GCC 4.5.0 ||
+        ln -s ../mpc-$VERSION_MPC $MAINSRCDIR/gcc-$VERSION_GCC/mpc)
   then
     log_error "Creation of links to additional gcc modules failed"
     exit 1
@@ -569,7 +585,7 @@ fi
 
 if test "x$CROSSER_DOWNLOAD" = "xyes"
 then
-  if ! ( cd $PACKETDIR && $MAINDIR/scripts/download_packets.sh "$STEPPARAM" )
+  if ! ( cd $PACKETDIR && $CROSSER_MAINDIR/scripts/download_packets.sh "$STEPPARAM" )
   then
     log_error "Download failed"
     exit 1
@@ -605,6 +621,7 @@ if test "x$STEP_NATIVE" = "xyes" ; then
      ! build_for_host binutils binutils-$VERSION_BINUTILS     \
      "--with-tls --enable-stage1-languages=all"               ||
      ! prepare_gcc_src                                        ||
+     ! log_write 1 "-Building native gcc can take hours!-"    ||
      ! build_for_host gcc gcc-$VERSION_GCC                    \
      "--enable-languages=c,c++ --disable-multilib --with-tls"
   then
@@ -766,17 +783,19 @@ then
       LIBCVER=$VERSION_GLIBC
       LIBCDIR=$LIBCNAME-$LIBCVER
       if ! unpack_component $LIBCNAME       $LIBCVER       ||
-         ! unpack_component $LIBCNAME-ports $LIBCVER $LIBCDIR
+         ! unpack_component $LIBCNAME-ports $VERSION_GLIBC_PORTS $LIBCDIR
       then
         crosser_error "$LIBCNAME unpacking failed"
         exit 1
       fi
-      if ! ln -s $LIBCNAME-ports-$LIBCVER $MAINSRCDIR/$LIBCDIR/ports
+      if ! ln -s $LIBCNAME-ports-$VERSION_GLIBC_PORTS $MAINSRCDIR/$LIBCDIR/ports
       then
 	crosser_error "Creation of $LIBCNAME links failed"
         exit 1
       fi
-      if ! patch_src $LIBCDIR glibc_binutils2.20
+      if ! ( is_minimum_version $LIBCVER 2.11 ||
+             patch_src $LIBCDIR glibc_binutils2.20 ) ||
+         ! patch_src $LIBCDIR glibc_i686define
       then
         crosser_error "$LIBCNAME patching failed"
         exit 1
@@ -876,17 +895,51 @@ then
   if ! patch_src zlib               zlib_cctest               ||
      ! patch_src zlib               zlib_seeko
   then
-    error_log "Baselib patching failed"
+    crosser_error "Baselib patching failed"
     exit 1
   fi
 
-  if ! build_zlib                zlib    zlib                 \
-        "--shared"                                            ||
+  if ! build_zlib                zlib    zlib                     \
+        "--shared"                                                ||
      ! build_with_cross_compiler libpng  libpng-$VERSION_PNG
   then
     crosser_error "Baselib build failed"
     exit 1
   fi
+
+  if test "x$E_GLIBC" != "xyes"
+  then
+    log_write 1 "Some parts of baselib step not available for $LIBC_MODE based builds, skipping"
+  else
+    if ! unpack_component libxml2 $VERSION_LIBXML2                  ||
+       ! build_with_cross_compiler libxml2 libxml2-$VERSION_LIBXML2
+    then
+      crosser_error "Baselib build failed"
+      exit 1
+    fi
+  fi
+fi
+
+if test "x$STEP_XORG" = "xyes"
+then
+  STEP="xorg"
+  STEPADD="    "
+
+  if test "x$E_GLIBC" != "xyes"
+  then
+    log_write 1 "Step xorg not available for $LIBC_MODE based builds, skipping"
+  else
+    if ! unpack_component xproto $VERSION_XORG_XPROTO                          ||
+       ! build_with_cross_compiler xproto xproto-$VERSION_XORG_XPROTO          ||
+       ! unpack_component xextproto $VERSION_XORG_XEXTPROTO                    ||
+       ! build_with_cross_compiler xextproto xextproto-$VERSION_XORG_XEXTPROTO ||
+       ! unpack_component xtrans $VERSION_XORG_XTRANS                          ||
+       ! build_with_cross_compiler xtrans xtrans-$VERSION_XORG_XTRANS
+    then
+      crosser_error "Xorg build failed"
+      exit 1
+    fi
+  fi
 fi
 
 if test "x$STEP_GTK" = "xyes"
