@@ -2,18 +2,37 @@
 
 # trim_packetdir.sh: Delete old source packets
 #
-# (c) 2009 Marko Lindqvist
+# (c) 2009-2010 Marko Lindqvist
 #
 # This program is licensed under Gnu General Public License version 2.
 
 export FILELIMIT=200
 
+CROSSER_MAINDIR="$(cd $(dirname $0)/.. ; pwd)"
+
+if ! test -e "$CROSSER_MAINDIR/CrosserVersion"
+then
+  CROSSER_MAINDIR="/usr/share/crosser"
+fi
+
+if ! . $CROSSER_MAINDIR/scripts/helpers.sh
+then
+  echo "Failed to read $CROSSER_MAINDIR/scripts/helpers.sh" >&2
+  exit 1
+fi
+
 if test "x$1" = "x-h" || test "x$1" = "x--help" ; then
   HELP_RETURN=0
 elif test "x$1" = "x" ; then
   HELP_RETURN=1
 fi
 
+if test "x$1" = "x-v" || test "x$1" = "x--version"
+then
+  echo "Packetdir trimmer script for Crosser $CROSSER_VERSION"
+  exit 0
+fi
+
 if test "x$HELP_RETURN" != "x" ; then
   echo "Usage: $(basename $0) <packetdir>"
   exit $HELP_RETURN
