@@ -17,6 +17,8 @@ do
       STEP_BASELIB=yes ;;
     win)
       STEP_WIN=yes ;;
+    xorg)
+      STEP_XORG=yes ;;
     gtk)
       STEP_GTK=yes ;;
     sdl)
